-- DOS font tools

without warning
without trace

include machine.e     -- We need to access BIOS interrupts
include graphics.e  


----------------------------------------------------------------------------
-- define the controls
-- running load_graphic_font redefines these
global sequence CHECK_ON, CHECK_OFF, RADIO_ON, RADIO_OFF
    -- set the special characters
    RADIO_ON    = '(' & 7 & ')'
    RADIO_OFF   = "( )"
    CHECK_ON    = "[X]"
    CHECK_OFF   = "[ ]"


----------------------------------------------------------------------------
procedure add_font( integer char, sequence pattern )

    -- move a sequence of fonts into memory

    object    lowMem
    sequence  inReg, outReg

    lowMem = allocate_low( length( pattern ) )   -- get memory
    poke( lowMem, pattern )         -- move description to memory

    inReg = repeat( 0, 10 )         -- initialize register list
    inReg[REG_AX] = #1100           -- new char function
    inReg[REG_BX] = 16 * 256        -- 16 bytes/char - for 256 characters
    inReg[REG_CX] = 1               -- number of characters to change
    inReg[REG_DX] = char            -- first character to change
    inReg[REG_ES] = floor(lowMem / 16)    -- segment of low_mem (low memory)
    inReg[REG_BP] = remainder(lowMem, 16) -- offset of low_mem
    outReg = dos_interrupt( #10, inReg )  -- bios call

    free_low( lowMem )  -- free the memory

end procedure


----------------------------------------------------------------------------
global procedure font( integer char, sequence pattern )
    
    -- load a font file into memory
    integer accum
    
    -- replace pattern with bits
    for i = 1 to 16 do
        
        -- convert to binary
        accum = 0
        if pattern[i][1] = 'x' then accum = accum + 128       end if
        if pattern[i][2] = 'x' then accum = accum + 64        end if
        if pattern[i][3] = 'x' then accum = accum + 32        end if
        if pattern[i][4] = 'x' then accum = accum + 16        end if
        if pattern[i][5] = 'x' then accum = accum + 8         end if
        if pattern[i][6] = 'x' then accum = accum + 4         end if
        if pattern[i][7] = 'x' then accum = accum + 2         end if
        if pattern[i][8] = 'x' then accum = accum + 1         end if
        
        -- replace with number
        pattern[i] = accum
        
    end for

    add_font( char, pattern )

end procedure


----------------------------------------------------------------------------
global procedure reset_font()
    
    -- reset the fonts to the default BIOS

    sequence  inReg, outReg             -- registers
    inReg = repeat( 0, 10 )             -- initialize register list
    inReg[REG_AX] = #3                  -- BIOS reset the video display to 80
                                        -- cols x 25 lines
    outReg = dos_interrupt( #10, inReg )  -- bios call

end procedure


constant
    SCREEN_WIDE     = 80,
    SCREEN_DEEP     = 25,
    SCREEN_MEMORY   = #B8000

----------------------------------------------------------------------------
global procedure put_colors( sequence colors, object text )
        
    -- given a line of text, or sequence of lines, writes them to the
    -- screen with the given attributes

    sequence cursorAt
    integer thisRow, thisCol, bpl
    integer attribute, pointer

    -- convert to a list if text is a single byte
    -- this is to save me from myself
    if atom( text ) then
            text = { text }
    end if

    -- skip if empty
    if length( text ) = 0 then
            return
    end if

    -- get cursor location
    cursorAt = get_position()
    thisRow = cursorAt[1]
    thisCol = cursorAt[2]

    -- turn off screen wrapping
    wrap( 0 )

    -- if single line, convert to a list
    if atom( text[1] ) then

        -- make it an item in a list
        text = { text }

    end if

    -- combine the colors into a single attribute byte
    attribute = ( colors[2] * #10 ) + colors[1]

    -- video is located at HB8000 for color, and HB0000 for monochrome

    -- bytes per line
    bpl = SCREEN_WIDE*2

    for row = 1 to length( text ) do
        -- calculate offset onto screen (line, col)
        pointer = SCREEN_MEMORY             -- start of video
            + ( (thisRow+row-2) * bpl )     --  bytes per line
            + ( (thisCol-1) * 2 )           -- 2 bytes per line

        for col = 1 to length( text[row] ) do
            poke( pointer, text[row][col] )
            poke( pointer+1, attribute )
            pointer = pointer + 2
        end for
    end for

    -- new row
    thisRow = thisRow + length(text) - 1
    if thisRow > SCREEN_DEEP then 
        thisRow = SCREEN_DEEP 
    end if

    -- new column
    thisCol = thisCol + length( text[length(text)] )
    if thisCol > SCREEN_WIDE then 
        thisCol = SCREEN_WIDE 
    end if

    -- reposition cursor
    position( thisRow, thisCol )


end procedure


-----------------------------------------------------------------------------
global procedure put_shadow( integer row, integer col, integer wide )
        
    -- draw a shadow at the current position

    integer bpl
    integer attribute, pointer

    -- combine the colors into a single attribute byte
    attribute = ( BLACK * #10 ) + GRAY

    -- bytes per line 
    bpl = SCREEN_WIDE * 2

    -- poke into video

    -- calculate offset onto screen (line, col)
    pointer = SCREEN_MEMORY     -- start of video
        + ( (row-1) * bpl )     -- SCREEN_WIDE*2 bytes per line
        + ( (col-1) * 2 )       -- 2 bytes per line

    -- poke it in
    for i = 1 to wide do
        poke( pointer+1, attribute )
        pointer = pointer + 2
    end for

    -- new column
    col = col + wide
    if col > SCREEN_WIDE then col = SCREEN_WIDE end if

    -- reposition cursor
    position( row, col )

end procedure


----------------------------------------------------------------------------
procedure toggle_blinking( integer flag )

    -- set text mode interpretation of high background colors
    -- 0: colors are high intensity
    -- 1: colors are blinking

    sequence  inReg, outReg

    inReg = repeat( 0, 10 )         -- initialize register list
    inReg[REG_AX] = #1003           -- function 10, subfunction 03
    inReg[REG_BX] = flag            -- enable/disable intensity
    outReg = dos_interrupt( #10, inReg )  -- bios call

end procedure

----------------------------------------------------------------------------
-- turn on intense colors
        toggle_blinking(0)

----------------------------------------------------------------------------
global procedure graphic_font()

    -- loads the graphic font
    -- this adds radio buttons and check buttons
    -- it also redefines CHECK_ON/OFF and RADIO_ON/OFF

    -- turn on intense colors
        toggle_blinking(0)

    -- graphic fonts
    -- right of radio button
        font(181, {
        "........",
        "........",
        "........",
        "x.......",
        "x.......",
        ".x......",
        ".x......",
        ".x......",
        ".x......",
        "x.......",
        "x.......",
        "........",
        "........",
        "........",
        "........",
        "........"} )
    -- left of check box 
        font( 198, {
        "........",
        ".....xxx",
        ".....xx.",
        ".....xx.",
        ".....xx.",
        ".....xx.",
        ".....xx.",
        ".....xx.",
        ".....xx.",
        ".....xx.",
        ".....xx.",
        ".....xx.",
        ".....xxx",
        ".......x",
        "........",
        "........"} )
    -- middle of checked check box
        font( 206, {
        "........",
        "xxxxxxxx",
        "........",
        ".....xx.",
        ".....xx.",
        "....xx..",
        "....xx..",
        "x..xx...",
        "xx.xx...",
        "xxxx....",
        ".xxx....",
        "........",
        "xxxxxxxx",
        "xxxxxxxx",
        "........",
        "........"} )
    -- middle of unchecked check box
        font( 208, {
        "........",
        "xxxxxxxx",
        "........",
        "........",
        "........",
        "........",
        "........",
        "........",
        "........",
        "........",
        "........",
        "........",
        "xxxxxxxx",
        "xxxxxxxx",
        "........",
        "........"} )
    -- left of radio button
        font ( 213, {
        "........",
        "........",
        ".......x",
        "......x.",
        "......x.",
        ".....x..",
        ".....x..",
        ".....x..",
        ".....x..",
        "......x.",
        "......x.",
        ".......x",
        "........",
        "........",
        "........",
        "........"} )
    -- middle of unchecked radio button
        font( 214, {
        "........",
        "xxxxxxx.",
        ".......x",
        "........",
        "........",
        "........",
        "........",
        "........",
        "........",
        "........",
        "........",
        ".......x",
        "xxxxxxx.",
        "........",
        "........",
        "........"} ) 
    -- middle of checked radio button
        font ( 216, {
        "........",
        "xxxxxxx.",
        ".......x",
        "........",
        ".xxxxx..",
        "xxxxxxx.",
        "xxxxxxx.",
        "xxxxxxx.",
        "xxxxxxx.",
        ".xxxxx..",
        "........",
        ".......x",
        "xxxxxxx.",
        "........",
        "........",
        "........"} )
    -- right of check box
        font( 227, {
        "........",
        "xxx.....",
        ".xx.....",
        ".xxx....",
        ".xxx....",
        ".xxx....",
        ".xxx....",
        ".xxx....",
        ".xxx....",
        ".xxx....",
        ".xxx....",
        ".xxx....",
        "xxxx....",
        "xxxx....",
        "........",
        "........"} )
    -- replacement check
        font( 251, {
        "........",
        ".......x",
        "......xx",
        "......x.",
        ".....xx.",
        ".....x..",
        "....xx..",
        "x...x...",
        "xx.xx...",
        ".x.x....",
        ".xxx....",
        "..x.....",
        "........",
        "........",
        "........",
        "........"} )

    -- set the special characters
    CHECK_ON    = 198 & 206 & 227
    CHECK_OFF   = 198 & 208 & 227
    RADIO_ON    = 213 & 216 & 181
    RADIO_OFF   = 213 & 214 & 181
        
end procedure


